local Addon, TA = ...

-- Generic Locals
local on = true
local off = false
local PerSec = 8 -- run the Queue scan X times a second
local AnnounceDupeTime = 1 -- Duplicate announce filter time (if same spell happens within x seconds by same person it won't announce it)
local ToTell = false -- flag if something to tattle happens

-- Addon Settings
TauntAware_Settings = {}
local TAS = nil 
TauntAware_Default_Settings = {
	Enable = on,
	Full = on,
	TauntsOnly = on,
	Links = on,
	Scan = on,
	Tattle = off,
	TatMode = "GROUP", -- either WHISPER, GROUP, SUMMARY (summary just prints to you)
}


-- Bill's Utils
local SJprint = BillsUtils.SJprint
local addOptionMt = BillsUtils.addOptionMt
local StatColor = BillsUtils.StatColor
local OptSaveTF = BillsUtils.OptSaveTF
local CCC = BillsUtils.CCC

BillsUtils.Locals[#BillsUtils.Locals +1] = function ()
	SJprint = BillsUtils.SJprint
	addOptionMt = BillsUtils.addOptionMt
	StatColor = BillsUtils.StatColor
	OptSaveTF = BillsUtils.OptSaveTF
	CCC = BillsUtils.CCC
end

-- Local text colors
local yel = "|cFFFFFF00"
local wht = "|cFFFFFFFF"
local blu = "|cFF00FFFF"
local red = "|cFFFF0000"
local grn = "|cFF00FF00"
local res = "|r"

local SpellList = {}
-- SpellId 

SpellList = {
	--DRUID
	--SPELL_CAST_SUCCESS
	[6795] = true,	-- Growl
	[5209] = true,	-- Challenging Roar

	--HUNTER
	--SPELL_CAST_SUCCESS
	[34477] = true,	-- Misdirection
	[20736] = true,	-- Distracting Shot

	--MAGE

	--PALADIN
	--SPELL_CAST_SUCCESS
	[31789] = true,	-- Righteous Defense
	[62124] = true,	-- Hand of Reckoning
	[25780] = true,	-- Righteous Fury

	--PRIEST
	
	--ROGUE
	--SPELL_CAST_SUCCESS
	[57934] = true,	-- Tricks of the Trade

	--SHAMAN
	--SPELL_CAST_SUCCESS
	[8056] = true,	-- Frost Shock
	--"SPELL_AURA_APPLIED"
	[73684] = true,	-- Unleash Earth

	--WARLOCK

	--WARRIOR
	--SPELL_CAST_SUCCESS
	[355] = true,	-- Taunt
	[1161] = true,	-- Fury
	[57755] = true,	-- Heroic Throw
	[71] = true,	-- Defensive Stance

	--DEATHKNIGHT
	--SPELL_CAST_SUCCESS
	[48263] = true,	-- Blood Presence
	[49576] = true,	-- Death Grip
	[56222] =  true,	-- Dark Command

	--PETS
	-- Warlock Pets
	[17735] = true,	-- Suffering
	[3716] = true,	-- Torment
	-- Hunter Pets
	[2649] = true,	-- Growl
	[53477] = true,	-- Taunt
	[24394] = true,	-- Intimidation
	[63900] = true,	-- Thunderstomp
}

local ScanSpells = {
	[25780] = true,	-- Pally	Righteous Fury
	[48263] = true,	-- DK		Blood Presence
	[71] = true,	-- War 		Defensive Stance
}

local LastPartyMembers ={}
TA.ScanGroup = function( UserCalled )
	local PNum = GetNumPartyMembers()
	local RNum = GetNumRaidMembers()
	local Type = ""
	local Count = 0
			
	if RNum > 0 then
		Type = "raid"
		Count = RNum
	elseif PNum > 0 then
		Type = "party"
		Count = PNum
	else
		return
	end
	
	local changed = false
	
	for x = 1, count do
		if LastPartyMembers[x] ~= UnitName(Type..tostring(x)) then
			changed = true
		end
	end
		
	if not(changed) and not(UserCalled) then
		return
	end
	
	table.wipe(LastPartyMembers)
	
	for x = 1, Count do
		LastPartyMembers[x] = UnitName(Type..tostring(x))
		local role = UnitGroupRolesAssigned(Type..tostring(x))
		if role ~= "TANK" then
			for y = 1, 40 do
				local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura( Type..tostring(x), y)
				if not spellId then
					break
				end
				if ScanSpells[spellId] then
					local _, class = UnitClass(Type..tostring(x))
					local charName = UnitName(Type..tostring(x))
					if TAS.Links then
						print( ("%s<Taunt Aware> %s|Hplayer:%s|h%s|h|r has |cff71d5ff|Hspell:%s|h[%s]|h|r on"):format(yel, CCC(class), charName, charName, spellId, name) )
					else
						print( ("%s<Taunt Aware> %s%s|r has |cff71d5ff%s|r on"):format(yel, CCC(class), charName, name) )
					end
				end
			end
		end
	end
	
	if Type == "party" then
		local role = UnitGroupRolesAssigned("player")
		if role ~= "TANK" then
			for y = 1, 40 do
				local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura( "player" , y)
				if not spellId then
					break
				end
				if ScanSpells[spellId] then
					local _, class = UnitClass("player")
					local charName = UnitName("player")
					if TAS.Links then
						print( ("%s<Taunt Aware> %s|Hplayer:%s|h%s|h|r has |cff71d5ff|Hspell:%s|h[%s]|h|r on"):format(yel, CCC(class), charName, charName, spellId, name) )
					else
						print( ("%s<Taunt Aware> %s%s|r has |cff71d5ff%s|r on"):format(yel, CCC(class), charName, name) )
					end
				end
			end
		end
	end
	
	print( ("%s<Taunt Aware>|r %s scan complete"):format(yel, Type) )
end

local scanTool = CreateFrame( "GameTooltip", "ScanTooltip", nil, "GameTooltipTemplate" )
scanTool:SetOwner( WorldFrame, "ANCHOR_NONE" )
local scanText = _G["ScanTooltipTextLeft2"] -- This is the line with <[Player]'s Pet>

function getPetOwner(petName)
	scanTool:ClearLines()
	scanTool:SetUnit(petName)
	local ownerText = scanText:GetText()
	if not ownerText then return nil end
	local owner, _ = string.split("'",ownerText) -- english
	if UnitExists(owner) then return owner end
	_, owner = string.split(" ", ownerText) -- russian
	if UnitExists(owner) then return owner end
	_, _, owner = string.split(" ", ownerText) -- german
	if UnitExists(owner) then return owner end
	
	return false -- This is for error catching 
end

local Queue = {}

TA.Frame = CreateFrame("Frame")
TA.Frame:RegisterEvent("ADDON_LOADED")
TA.Frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
TA.Frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
TA.Frame:RegisterEvent("RAID_ROSTER_UPDATE")

TA.Frame:SetScript("OnEvent",function(self, event, ...)
	local arg = ...
	if event == "ADDON_LOADED" and arg == Addon then
		SJprint(yel, "Taunt Aware v",red, GetAddOnMetadata(Addon, "Version"), yel, "loaded", res)
		addOptionMt( TauntAware_Settings, TauntAware_Default_Settings )
		TAS = TauntAware_Settings
		return self, event, ...
	end
	if not(TAS.Enable) then return end
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		Queue[#Queue +1] = { ... }
		return
	end
	if (event == "PARTY_MEMBERS_CHANGED" or event == "RAID_ROSTER_UPDATE") and TAS.Scan then
		TA.ScanGroup()
		return
	end	
	return
end)

local e = 0
local last = {}
local TattleQueue = {}
local TauntsOnly = true

TA.Frame:SetScript("OnUpdate", function(self, elapsed)
	e = e + elapsed
	if e < 1/PerSec then
		return
	end
	e = 0
	
	if not(TAS.Enable) then return end
	
	if TauntsOnly ~= TAS.TauntsOnly then
		if TAS.TauntsOnly then
			TauntsOnly = true
			SpellList[34477] = false	-- Hunter	Misdirection
			SpellList[57934] = false	-- Rogue	Tricks of the Trade
			SpellList[8056] = false	-- Shaman	Frost Shock
		else
			TauntsOnly = false
			SpellList[34477] = true	-- Hunter	Misdirection
			SpellList[57934] = true	-- Rogue	Tricks of the Trade
			SpellList[8056] = true	-- Shaman	Frost Shock
		end
	
	end
	
	while #Queue > 0 do
		local timestamp, eventType, hidecaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2, spellId, spellName = unpack(Queue[1])
		local PlayerRole = UnitGroupRolesAssigned("player")
		if UnitExists(sourceName) and SpellList[spellId] and ( eventType == "SPELL_CAST_SUCCESS" or eventType == "SPELL_AURA_APPLIED") then
			local B = tonumber(sourceGUID:sub(5,5), 16);
			local maskedB = B % 8; -- x % 8 has the same effect as x & 0x7 on numbers <= 0xf
			-- local knownTypes = {[0]="player", [3]="NPC", [4]="pet", [5]="vehicle"};
	
			local Self = UnitIsUnit(sourceName, "player")
			local SelfPet = UnitIsUnit(sourceName, "pet")
			local PC = maskedB == 0 
			local Pet = maskedB == 4
			
			local Announce = false
			if TAS.Full or PlayerRole == "TANK" then
				Announce = true
			elseif PlayerRole == "DAMAGER" and (Self or SelfPet) then
				SJprint(red, "Are You supposed to be taunting?")
				Announce = true
			elseif PlayerRole == "HEALER" then
				--whatever for healers
			elseif PlayerRole == "NONE" then
				Announce = true
			end
			
			--Duplicate announce check
			if Announce and last.sourceName == sourceName and last.spellId == spellId and (timestamp - last.timestamp < AnnounceDupeTime ) then
				Announce = false
			end
				
			if Announce then
				last.sourceName = sourceName
				last.spellId = spellId
				last.timestamp = timestamp
				
				local _, sourceClass = UnitClass(sourceName)
				local name = ""
				
				if Pet and TAS.Links then
					spellName = "|cff71d5ff|Hspell:"..spellId.."|h["..spellName.."]|h|r" -- Create Spell Link
					local PetOwner = getPetOwner(sourceName)
					if PetOwner then
						local _, PetOwnerClass = UnitClass( PetOwner )
						name = ("%s|Hplayer:%s|h[%s]|h %s%s "):format( CCC(PetOwnerClass), PetOwner, PetOwner, CCC(sourceClass), sourceName )
					else
						name = ("PET: %s%s "):format( CCC(sourceClass), sourceName )
					end
				elseif Pet then
					local PetOwner = getPetOwner(sourceName)
					if PetOwner then
						local _, PetOwnerClass = UnitClass( PetOwner )
						name = ("%s%s %s%s"):format( CCC(PetOwnerClass), PetOwner, sourceClass, sourceName )
					else
						name = ("PET: %s%s "):format( CCC(sourceClass), sourceName )
					end
				elseif TAS.Links then
					spellName = "|cff71d5ff|Hspell:"..spellId.."|h["..spellName.."]|h|r" -- Create Spell Link
					name = ("%s|Hplayer:%s|h[%s]|h"):format(CCC(sourceClass), sourceName, sourceName)
				else
					name = ("%s%s"):format(sourceClass, sourceName) 
				end
		
				local msg = ""
				
		
				if destName then
					msg = string.format("|cFFFFFF00<Taunt Aware> %s|cFFFFFFFF casts %s on |cFFFF0000%s|r", name, spellName, destName )
				else
					msg = string.format("|cFFFFFF00<Taunt Aware> %s|cFFFFFFFF casts %s|r", name, spellName )
				end
		
				print( msg )
				
				if TAS.Tattle then 
					if Pet or UnitGroupRolesAssigned( sourceName ) ~= "TANK" then
						ToTell = true
						if TattleQueue[sourceName] then
							if TattleQueue[sourceName][spellId] then
								TattleQueue[sourceName][spellId] = TattleQueue[sourceName][spellId] + 1
							else
								TattleQueue[sourceName][spellId] = 1
							end
						else
							TattleQueue[sourceName] = {}
							TattleQueue[sourceName][spellId] = 1
						end
					end
				end
			end
		end
		table.remove(Queue, 1)
		if TAS.Tattle and ToTell and not( InCombatLockdown() ) then
			local ChatType = false
						
			if TAS.TatMode == "GROUP" then
				if GetNumRaidMembers() > 0 then
					ChatType = "RAID"
				elseif GetNumPartyMembers() > 0 then
					ChatType = "PARTY"
				end
			elseif TAS.TatMode == "WHISPER" then
				ChatType = "WHISPER"
			end
			
			for Name,Table in pairs(TattleQueue) do 
				for SpellId,Count in pairs(Table) do 
					local SpellLink = GetSpellLink(SpellId)
					local msg = ("<Taunt Aware> %s casted %s %s times last fight"):format( Name, SpellLink, Count )
					if ChatType then
						if ChatType == "PARTY" or ChatType == "RAID" then
							SendChatMessage( msg ,ChatType )
						elseif ChatType == "WHISPER" then
							local ChatTarget = ""
							-- Pet Check... Pets should not show up as in party/raid
							if UnitInParty(Name) or UnitInRaid(Name) or UnitIsUnit("player", Name) then
								ChatTarget = Name
							else
								ChatTarget = getPetOwner( Name )
							end	
							if ChatTarget then
								SendChatMessage( msg, ChatType, nil, ChatTarget)
							end
						else
							print(msg)
						end
					else
						print(msg)
					end
				end
			end
			table.wipe(TattleQueue)
			ToTell = false
		end
	end
	
end)

TA.CmdLine = function( msg )
	local cmd,arg = string.split(" ", msg)
	cmd = cmd:lower()
	
--	if cmd =="config" then
--		InterfaceOptionsFrame_OpenToCategory( TA.panel )
--		return
--	end
	
	if not(TAS.Enable) and cmd ~= "enable" then
		print(" ")
		SJprint( yel, "Taunt Aware", res)
		SJprint( yel, "  This addon is ", red, "DISABLED", res)
		SJprint( yel, "  Type: ", blu, "/ta enable on", res)
		SJprint( yel, "  to enable.", res)
		return
	end
	
	if cmd == "scannow" then
		TA.ScanGroup(true)
		return
	end
	
	--On/Off True/False values
	local Command = {"Enable", "Full", "TauntsOnly", "Links", "Scan", "Tattle",}
	
	for x = 1, #Command do
		if cmd == string.lower(Command[x]) then
			OptSaveTF( TAS, Command[x], arg)
			SJprint(  yel, "Taunt Aware:", Command[x],"is", StatColor( TAS[Command[x]] ), res)
			return
		end
	end
	
	if cmd == "tatmode" then
		if arg then
			local Mode = string.upper(arg)
			if Mode == "GROUP" or Mode == "WHISPER" or Mode == "SUMMARY" then
				TAS.TatMode = Mode
			else
				SJprint(  yel, "Taunt Aware:", red, Mode, wht, "is not a valid Tattle Mode")
				SJprint(  yel, "Taunt Aware:", wht, "Valid modes are", blu, "GROUP   WHISPER   SUMMARY")
				return
			end
		end
		SJprint(yel, "Taunt Aware:", blu,"TatMode", wht, "is", grn, TAS.TatMode, res)
		return
	end
	
	if cmd == "" then
		print(" ")
		SJprint( yel, "________________________________________", res)
		SJprint( yel, "Taunt Aware", res)
		SJprint( blu, "     Enable", wht, "is", StatColor(TAS.Enable), res)
		SJprint( blu, "     Full", wht, "alerts are", StatColor(TAS.Full), res)
		SJprint( blu, "     TauntsOnly", wht, "alerts are", StatColor(TAS.TauntsOnly), res)
		SJprint( blu, "     Links ", wht, "for abilities are", StatColor(TAS.Links), res)
		SJprint( blu, "     Scan ", wht, "on party change is", StatColor(TAS.Scan), res)
		SJprint( blu, "     Tattle ", wht, "on non tank taunts is", StatColor(TAS.Tattle), res)
		SJprint( blu, "     TatMode ", wht, "What channel you want to tattle to (GROUP   WHISPER   SUMMARY) is", grn, (TAS.TatMode), res)
		SJprint( yel, "Available commands are listed above in", blu, "BLUE.", res)
		SJprint( yel, "Type", blu, "/ta scannow", yel, "or", blu, "/tascan", yel, "to run a group scan",  res)
		SJprint( yel, "Type", blu, "/ta help", yel, "or", blu, "/ta cmdlist", yel, "for an explanation of the commands",  res)
--		SJprint( yel, "Type", blu, "/ta config", yel, "to open the GUI config screen.", res)
		SJprint( yel, "________________________________________", res)
		return
	end

	if cmd == "help" or cmd == "cmdlist" then
		print(" ")
		SJprint( yel, "________________________________________", res)
		SJprint( yel, "Taunt Aware", res)
		SJprint( blu, "     Enable", wht, "Enable the addon to function [on/off]", res)
		SJprint( blu, "     Full", wht, "Alerts no matter your role [on/off]", res)
		SJprint( blu, "     TauntsOnly", wht, "only alert on taunts, not on high threat actions [on/off]", res)
		SJprint( blu, "     Links ", wht, "enable spell/ player links when able [on/off]", res)
		SJprint( blu, "     Scan ", wht, "Scan the non tank group members for threat increasing abilities on a group change [on/off]", res)
		SJprint( blu, "     Tattle ", wht, "on non tank taunts [on/off]", res)
		SJprint( blu, "     TatMode ", wht, "What channel you want to tattle to (GROUP   WHISPER   SUMMARY)", res)
		SJprint( yel, "Available commands are listed above in", blu, "BLUE.", res)
		SJprint( yel, "Type", blu, "/ta scannow", yel, "or", blu, "/tascan", yel, "to run a group scan",  res)
--		SJprint( yel, "Type", blu, "/ta config", yel, "to open the GUI config screen.", res)
		SJprint( yel, "________________________________________", res)
	end
	
	print(" ")
	SJprint( yel, "  Taunt Aware", res)
	SJprint( yel, "  Error: ", red, cmd, yel, " is not a valid command", res)
	SJprint( yel, "  Type: ", blu, "/ta", yel, "for the addon status and command list", res)
	SJprint( yel, "  Type: ", blu, "/ta cmdlist", yel, "for command list help", res)
	return
end


SLASH_TA1 = "/ta"
SlashCmdList["TA"] = function(msg)
	TA.CmdLine(msg)
end

SLASH_TASCAN1 = "/tascan"
SlashCmdList["TASCAN"] = function(msg)
	TA.ScanGroup(true)
end